// -- a single match of length 12 at chosen positions in a 19-mer,
//  allowing one transition 
// -- a single match of length 11 at chosen positions in a 18-mer,
//  allowing one transition 


#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>

#define EXTRACT64(n,spos,len,dpos) \
    ((((n) >> (spos)) & ((1ULL<<(len))-1)) << (dpos))

#define EXTRACT32(n,spos,len,dpos) \
    ((((n) >> (spos)) & ((1U<<(len))-1)) << (dpos))

#define EXTRACTBP(n,spos,len,dpos) EXTRACT32(n,spos*2,len*2,dpos*2)
    
static uint32_t map_19to12(uint64_t i)
{
	// 0123456789012345678
	// 1110100110010101111 (12 of 19 bp)
	// 0   3  4   6 7 8
	return
	    EXTRACTBP(i, 0,3,0) |
	    EXTRACTBP(i, 4,1,3) |
	    EXTRACTBP(i, 7,2,4) |
	    EXTRACTBP(i,11,1,6) |
	    EXTRACTBP(i,13,1,7) |
	    EXTRACTBP(i,15,4,8) ;
}

static uint32_t map_19to12_1(uint64_t i)
{
	// 0123456789012345678 (bp)
	// 1110100110010101111 (12 of 19)
	// 0   3  4   6 7 8
	register const uint32_t a = i & 0xFFFFFFFF;
	register const uint32_t b = i >> 32;
	return
	    EXTRACTBP(a, 0,3,0) |
	    EXTRACTBP(a, 4,1,3) |
	    EXTRACTBP(a, 7,2,4) |
	    EXTRACTBP(a,11,1,6) |
	    EXTRACTBP(a,13,1,7) |
	    EXTRACTBP(a,15,1,8) |
	    EXTRACTBP(b, 0,3,0) ;
}

static uint32_t map_18to11(uint64_t i)
{
	// 012345678901234567
	// 111010010100110111  (11 of 18 bp)
	// 0   3  4 5  6  8
	return
	    EXTRACTBP(i, 0,3,0) |
	    EXTRACTBP(i, 4,1,3) |
	    EXTRACTBP(i, 7,1,4) |
	    EXTRACTBP(i, 9,1,5) |
	    EXTRACTBP(i,12,2,6) |
	    EXTRACTBP(i,15,3,8) ;
}

uint8_t m[256];
static uint32_t map_19to12_s(const uint8_t *s)
{
	uint32_t i;
	i = m[s[0]];
	i <<= 2; i |= m[s[1]];
	i <<= 2; i |= m[s[2]];
	i <<= 2; i |= m[s[4]];
	i <<= 2; i |= m[s[7]];
	i <<= 2; i |= m[s[8]];
	i <<= 2; i |= m[s[11]];
	i <<= 2; i |= m[s[13]];
	i <<= 2; i |= m[s[15]];
	i <<= 2; i |= m[s[16]];
	i <<= 2; i |= m[s[17]];
	i <<= 2; i |= m[s[18]];
	return i;
}


int main()
{
    uint64_t a,b,c,d,e;
    int i, j, k;
    char *s;

#if 0
    a = in();
    i = EXTRACT32(a,12,2,6);
    out(i);
#endif

#if 0
    a = in();
    i = map_19to12(a);
    out(i);
#endif

#if 1
    in(s);
    i = map_19to12_s(s);
    out(i);
#endif

#if 0
    a = 0x123456789ABCDEF1;
    k = 36;

    printf("%llx\n", a);
    for (j=0; j<64; j+=k) {
	printf("%d: ", j);
	for (i=0; i<64; i+=4) {
	    printf("%llx ", EXTRACT64(a,j,k,i));
	}
	putchar('\n');
    }
#endif
    exit(0);
}
