/*
* dotprep.c - prepare a blastz output file for making a dotplot.
*
* The needed operations are (1) replace the stated length of sequence 2 by the
* sum of the lengths for all contigs in sequence 2, (2) identify all alignments
* to a reverse complement and modify associated diagonal lines so they run
* upper-left to lower-right, and (3) stack the dotplots for the contigs one on
* top of the other.
*/

#include "util.h"
#include "lav.h"

enum {
  BUF_SIZE=500
};

int main(int argc, char **argv)
{
	FILE *fp;
	int flip, P1, P1old, L1, L2, L2old, from1, from2, to1, to2, pct;
	int offset;
	char buf[BUF_SIZE], name[BUF_SIZE];
	int s1from, s1to;
	int nth, nth_old, rc;
	int len_arg = 0;
	char *lav_arg = 0;

	argv0 = "dotprep";
	if (argc >= 3)
		len_arg = atoi(argv[2]);
	if (argc >= 2)
		lav_arg = argv[1];
	if (argc < 2 || argc > 3) 
		fatal("usage: blastz-output [len]");

	/* align_header(L1 = seq1len(argv[1]), seq2len(argv[1])); */
	L1 = seq1bounds(lav_arg, &s1from, &s1to);

	if (len_arg == 0)
		len_arg = seq2len(lav_arg);
	align_header4(s1from,s1to,1,len_arg);

	fp = alignopen(lav_arg);
	nth_old = P1old = L2old = offset = flip = 0;
	while (fgets(buf, BUF_SIZE, fp)) {
		if (same_string(buf, "s {\n")) {
			lav_seqname4(2,fp,buf,BUF_SIZE,name,&P1,&L2,&rc,&nth);
			flip = rc;
			if (nth != nth_old)
				offset += L2old;
			P1old = P1;
			L2old = L2;
			nth_old = nth;
		} else if (sscanf(buf, "  l %d %d %d %d %d",
			    &from1, &from2, &to1, &to2, &pct) == 5) {
			if (flip)
				printf("  l %d %d %d %d %d\n",
				    from1, L2-from2+1+offset,
				    to1, L2-to2+1+offset, pct);
			else
				printf("  l %d %d %d %d %d\n",
				   from1, from2+offset, to1, to2+offset, pct);
		}
	}
	printf("}\n");
	fclose(fp);
	return 0;
}
