/* mkmatrix - make a human-mouse or mouse-human scoring matrix */

#include "util.h"
#include "seq.h"

enum {
  BUF_SIZE=1000,
  SCALE=10,
};

float m37[4][4] = {
  { 6.32914, -1.56897, -8.43596, -9.95695 },
  { -0.0911161, 8.31954, -7.51047, -7.65746 },
  { -7.65746, -7.51047, 8.31954, -0.0911161 },
  { -9.95695, -8.43596, -1.56897, 6.32914 },
};
float m43[4][4] = {
  { 6.91959, -1.72691, -8.64189, -9.94613 },
  { -0.416852, 7.28826, -8.79032, -8.32387 },
  { -8.32387, -8.79032, 7.28826, -0.416852 },
  { -9.94613, -8.64189, -1.72691, 6.91959 },
};
float m50[4][4] = {
  { 7.40281, -1.53077, -8.23094, -9.09788 },
  { -0.699182, 6.53342, -9.12126, -8.62817 },
  { -8.62817, -9.12126, 6.53342, -0.699182 },
  { -9.09788, -8.23094, -1.53077, 7.40281 },
};

int f2i(float f)
{
	return (f > 0.0) ? SCALE*f+0.5 : SCALE*f-0.5;
}

void matcp(float from[4][4], float to[4][4])
{
	int i, j;

	for (i = 0; i < 4; ++i)
		for (j = 0; j < 4; ++j)
			to[i][j] = from[i][j];
}

int main(int argc, char **argv)
{
	SEQ *sp;
	int total_len, N, GC, i, pct, human, j, bad;
	uchar *seq;
	float m[4][4], f;

	argv0 = "mkmatrix";
	if (argc != 5)
		fatalf("args = -h digit seq1 seq2");

	if (strcmp(argv[1], "-h"))
		fatal("first argument must be '-h'");

	human = atoi(argv[2]);

	sp = seq_get(argv[3]);
	if (sp == NULL)
		fatal("First sequence given in improper format.");
	total_len = N = SEQ_LEN(sp);
	seq = SEQ_CHARS(sp);
	for (GC = i = 0; i < N; ++i)
		if (strchr("cgCG", seq[i]))
			++GC;
	seq_close(sp);

	sp = seq_get(argv[4]);
	if (sp == NULL)
		fatal("Second sequence given in improper format.");
	total_len += (N = SEQ_LEN(sp));
	seq = SEQ_CHARS(sp);
	for (i = 0; i < N; ++i)
		if (strchr("cgCG", seq[i]))
			++GC;
	seq_close(sp);

	pct = (100*GC + total_len/2)/total_len;

	if (pct <= 40)
		matcp(m37, m);
	else if (pct <= 46)
		matcp(m43, m);
	else
		matcp(m50, m);

	printf("a\tg\tc\tt\tA\tG\tC\tT\tX\n");
	bad = -SCALE * f2i(m[0][0]);
	for (i = 0; i < 8; ++i) {
		for (j = 0; j < 8; ++j) {
			f = (human ? m[i%4][j%4] : m[j%4][i%4]);
			printf("%d\t", f2i(f));
		}
		printf("%d\n", bad);
	}
	for (j = 0; j < 8; ++j)
		printf("%d\t", bad);
	printf("%d\n", bad);
	
	return 0;
}
