#include <stdio.h>
#include <stdlib.h>

struct ali {
    int a, b, z, y;
    float p;
};

typedef struct ali_array ali_array;
struct ali_array {
    struct ali *a;
    int n;
    int max;
};

typedef struct ali_idx_array ali_idx_array;
struct ali_idx_array {
    int *a;
    int n;
    int max;
};

typedef struct out_array out_array;
struct out_array {
    ali_idx_array *a;
    int n;
    int max;
};

enum { NOUT=30000 };

struct ali_array All;
struct ali_idx_array Out[NOUT];

char fake[] = 
"#:lav\n"
"d {\n"
"  \"split-align\"\n"
"}\n"
"s {\n"
"  \"fakeseq1\" 1 1\n"
"  \"fakeseq2\" 1 1\n"
"}\n"
"h {\n"
"   \">fakeheader1\"\n"
"   \">fakeheader2\"\n"
"}\n"
"a {\n"
"  s 0\n"
"  b 1 2\n"
"  e 1 2\n"
;

void *alloc(void *p, unsigned int n)
{
    p = realloc(p, n);
    if (!p) { fprintf(stderr, "alloc %u failed\n", n); exit(1); }
    return p;
}

ali_idx_array* Bucket(int n)
{
    if (n < 0 || n >= NOUT)
	{ fprintf(stderr, "Bucket index %d failed\n", n); exit(1); }
    return &Out[n];
}

int Idx(ali_idx_array *B, int n)
{
    if (n < 0 || n >= B->n)
	{ fprintf(stderr, "Idx index %d failed\n", n); exit(1); }
    return B->a[n];
}

struct ali *Ali(int n)
{
    if (n < 0 || n >= All.n)
	{ fprintf(stderr, "Ali index %d failed\n", n); exit(1); }
    return &All.a[n];
}

int new_ali(int a, int b, int z, int y, float p)
{
    struct ali *A;
    if (All.n+1 >= All.max) {
	All.max = All.max + All.max/8 + 1000;
	All.a = alloc(All.a, All.max*sizeof(struct ali));
    }
    A = &All.a[All.n];
    A->a = a; A->b = b; A->z = z; A->y = y; A->p = p;
    return All.n++;
}

int put_ali(int bucket, int p)
{
    ali_idx_array *B = Bucket(bucket);
    if (B->n+1 >= B->max) {
	B->max = B->max + B->max/8 + 1000;
	B->a = alloc(B->a, B->max*sizeof(int*));
    }
    B->a[B->n] = p;
    return B->n++;
}

int emit(FILE *fp, int bucket)
{
    int n;
    ali_idx_array *B = Bucket(bucket);
    fputs(fake, fp);
    for (n = 0; n < B->n; ++n) {
	int i = Idx(B,n);
	struct ali *A = Ali(i);
	fprintf(fp, "  l %d %d %d %d %g\n", A->a, A->b, A->z, A->y, A->p);
    }
    return 0;
}

char *bucketfile(char *buf, int bs, char *path, int n)
{
    if (snprintf(buf, bs, "%s/%d", path, n) < bs)
	return buf;
    fprintf(stderr, "buf too small (%d)\n", bs);
    exit(1);
}

int main(int argc, char **argv)
{
    char *dir;
    int start, end, step;
    int a, b, z, y;
    double p;
    int off1, off2;
    int n, i, k;
    char buf[8192];
    char tok[32];


    if (argc != 5) {
	fprintf(stderr, "usage: dir start end step\n");
	exit(1);
    }
    dir = argv[1];
    start = atoi(argv[2]);
    end = atoi(argv[3]);
    step = atoi(argv[4]);

    while (fgets(buf, sizeof buf, stdin)) {
	n = sscanf(buf, "%32s", tok);
	if (n == 1 && tok[0] == 's') {
	    if (!fgets(buf, sizeof buf, stdin)) abort();
	    if (1 != sscanf(buf, "%*s %d", &off1)) abort();
	    if (!fgets(buf, sizeof buf, stdin)) abort();
	    if (1 != sscanf(buf, "%*s %d", &off2)) abort();
	    continue;
	}

	n = sscanf(buf, "%32s %d %d %d %d %lg", tok,&a,&b,&z,&y,&p);
	if (n == 6 && tok[0] == 'l') {
	    k = (a-start)/step;
	    for (i=k*step+start; i<=z+step && i<=end; ++k, i+=step) {
		if (i <= z && a <= i+step) {
		    if (k >= 0) put_ali(k, new_ali(a,b,z,y,p));
		}
	    }
	}
    }

    z = (end-start)/step+1;
    if (z > NOUT) z = NOUT;
    for (n = 0; n < z; ++n) {
	//if (Bucket(n)->n > 0) {
	    int i = n*step+start;
	    FILE *fp = fopen(bucketfile(buf,sizeof buf,dir,i), "w");
	    if (!fp) { fprintf(stderr, "{%s}\n", buf); perror("fopen"); exit(1); }
	    fprintf(fp, "# %d %d %d\n", i, start, step);
	    emit(fp, n);
	    fclose(fp);
	//}
    }
    return 0;
}
