#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

struct line {
    int a, z;
    char *p;
};

typedef struct line_array line_array;
struct line_array {
    struct line *a;
    int n;
    int max;
};

typedef struct line_idx_array line_idx_array;
struct line_idx_array {
    int *a;
    int n;
    int max;
};

enum { NOUT=30000 };

struct line_array All;
struct line_idx_array Out[NOUT];

void *alloc(void *p, unsigned int n)
{
    p = realloc(p, n);
    if (!p) { fprintf(stderr, "alloc %u failed\n", n); exit(1); }
    return p;
}

line_idx_array* Bucket(int n)
{
    if (n < 0 || n >= NOUT)
	{ fprintf(stderr, "Bucket index %d failed\n", n); exit(1); }
    return &Out[n];
}

int Idx(line_idx_array *B, int n)
{
    if (n < 0 || n >= B->n)
	{ fprintf(stderr, "Idx index %d failed\n", n); exit(1); }
    return B->a[n];
}

struct line *Line(int n)
{
    if (n < 0 || n >= All.n)
	{ fprintf(stderr, "Line index %d failed\n", n); exit(1); }
    return &All.a[n];
}

int new_line(int a, int z, char *p)
{
    struct line *A;
    if (All.n+1 >= All.max) {
	All.max = All.max + All.max/8 + 1000;
	All.a = alloc(All.a, All.max*sizeof(struct line));
    }
    A = &All.a[All.n];
    A->a = a; A->z = z; A->p = strdup(p);
    return All.n++;
}

int put_line(int bucket, int p)
{
    line_idx_array *B = Bucket(bucket);
    if (B->n+1 >= B->max) {
	B->max = B->max + B->max/8 + 1000;
	B->a = alloc(B->a, B->max*sizeof(int*));
    }
    B->a[B->n] = p;
    return B->n++;
}

int emit(FILE *fp, int bucket)
{
    int n;
    line_idx_array *B = Bucket(bucket);
    for (n = 0; n < B->n; ++n) {
	int i = Idx(B,n);
	struct line *A = Line(i);
	fputs(A->p, fp);
	fputc('\n', fp);
    }
    return 0;
}

char *bucketfile(char *buf, int bs, char *path, int n)
{
    if (snprintf(buf, bs, "%s/%d", path, n) < bs)
	return buf;
    fprintf(stderr, "buf too small (%d)\n", bs);
    exit(1);
}

int empty(char *buf)
{
    if (!buf) return 1;
    while (isspace(*buf)) ++buf;
    return *buf == 0;
}

int isdir(char *dir)
{
    struct stat buf;
    if (stat(dir, &buf) < 0) {
	fprintf(stderr, "{%s}: ", dir);
	perror("stat");
	exit(1);
    }
    return S_ISDIR(buf.st_mode);
}

int main(int argc, char **argv)
{
    char *dir;
    int start, end, step;
    int a, z, overlap=0;
    int n;
    char buf[8192];

    if (argc != 5 && argc != 6) {
	fprintf(stderr, "usage: dir start end step [overlap]\n");
	exit(1);
    }
    dir = argv[1];
    start = atoi(argv[2]);
    end = atoi(argv[3]);
    step = atoi(argv[4]);
    if (argc == 6) {
	overlap=atoi(argv[5]);
    }

    if (!isdir(dir)) {
	fprintf(stderr, "{%s} is not a directory\n", dir);
	exit(1);
    }

    while (fgets(buf, sizeof buf, stdin)) {
	buf[strlen(buf)-1] = 0;
	if (empty(buf))
	    continue;
	n = sscanf(buf, "%d %d", &a,&z);
	//fprintf(stderr, "--> %d %d\n", a, z);
	if (n != 2) {
	    int i;
	    int k = 0;
	    for (i=start; i<=end; ++k, i+=step) {
		put_line(k, new_line(0,0,buf));
	    }
	} else {
	    int i;
	    int k = (a-start)/step;
	    if (k > 0) --k;
	    for (i=k*step+start; i<=z+step && i<=end; ++k, i+=step) {
		//fprintf(stderr, "%d,%d,%d  ", i,i+step,i+step+overlap);
		if (i <= z && a <= i+step+overlap) {
		    //fprintf(stderr, "! %d", k);
		    if (k >= 0) put_line(k, new_line(a,z,buf));
		}
		//fprintf(stderr, "\n");
	    }
	}
    }

    z = (end-start)/step+1;
    if (z > NOUT) z = NOUT;
    for (n = 0; n < z; ++n) {
	//if (Bucket(n)->n > 0) {
	    int i = n*step+start;
	    FILE *fp = fopen(bucketfile(buf,sizeof buf,dir,i), "w");
	    if (!fp) { perror("fopen"); exit(1); }
	    fprintf(fp, "%% %d %d %d %d %d\n", i, start, step, overlap, end);
	    emit(fp, n);
	    fclose(fp);
	//}
    }
    return 0;
}
